@@ -12,6 +12,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "soup-coding.h"
 #include "soup-connection.h"
 #include "soup-message.h"
 #include "soup-message-private.h"
@@ -192,21 +193,20 @@ io_error (SoupSocket *sock, SoupMessage *msg, GError *error)
 	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
 	SoupMessageIOData *io = priv->io_data;
 
-	if (io->mode == SOUP_MESSAGE_IO_CLIENT &&
-	    io->read_state <= SOUP_MESSAGE_IO_STATE_HEADERS &&
-	    io->read_meta_buf->len == 0 &&
-	    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT) &&
-	    request_is_idempotent (msg)) {
+	if (error && error->domain == SOUP_SSL_ERROR) {
+		soup_message_set_status_full (msg,
+					      SOUP_STATUS_SSL_FAILED,
+					      error->message);
+	} else if (io->mode == SOUP_MESSAGE_IO_CLIENT &&
+		   io->read_state <= SOUP_MESSAGE_IO_STATE_HEADERS &&
+		   io->read_meta_buf->len == 0 &&
+		   !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT) &&
+		   request_is_idempotent (msg)) {
 		/* Connection got closed, but we can safely try again */
 		priv->io_status = SOUP_MESSAGE_IO_STATUS_QUEUED;
-	} else if (!SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code)) {
-		if (error && error->domain == SOUP_SSL_ERROR) {
-			soup_message_set_status_full (msg,
-						      SOUP_STATUS_SSL_FAILED,
-						      error->message);
-		} else
-			soup_message_set_status (msg, SOUP_STATUS_IO_ERROR);
-	}
+	} else if (!SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code))
+		soup_message_set_status (msg, SOUP_STATUS_IO_ERROR);
+
 	if (error)
 		g_error_free (error);
 
@@ -342,9 +342,54 @@ read_metadata (SoupMessage *msg, gboolean to_blank)
 		}
 	}
 
+	if (soup_socket_is_ssl (io->sock)) {
+		gboolean trusted_certificate;
+
+		g_object_get (io->sock,
+			      SOUP_SOCKET_TRUSTED_CERTIFICATE, &trusted_certificate,
+			      NULL);
+
+		if (trusted_certificate)
+			soup_message_set_flags (msg, priv->msg_flags | SOUP_MESSAGE_CERTIFICATE_TRUSTED);
+	}
+
 	return TRUE;
 }
 
+static SoupBuffer *
+content_decode (SoupMessage *msg, SoupBuffer *buf)
+{
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupCoding *decoder;
+	SoupBuffer *decoded;
+	GError *error = NULL;
+	GSList *d;
+
+	for (d = priv->decoders; d; d = d->next) {
+		decoder = d->data;
+
+		decoded = soup_coding_apply (decoder, buf->data, buf->length,
+					     FALSE, &error);
+		if (error) {
+			if (g_error_matches (error, SOUP_CODING_ERROR, SOUP_CODING_ERROR_INTERNAL_ERROR))
+				g_warning ("Content-Decoding error: %s\n", error->message);
+			g_error_free (error);
+
+			soup_message_set_flags (msg, priv->msg_flags & ~SOUP_MESSAGE_CONTENT_DECODED);
+			break;
+		}
+		if (buf)
+			soup_buffer_free (buf);
+
+		if (decoded)
+			buf = decoded;
+		else
+			return NULL;
+	}
+
+	return buf;
+}
+
 /* Reads as much message body data as is available on io->sock (but no
  * further than the end of the current message body or chunk). On a
  * successful read, emits "got_chunk" (possibly multiple times), and
@@ -395,10 +440,14 @@ read_body_chunk (SoupMessage *msg)
 
 		if (status == SOUP_SOCKET_OK && nread) {
 			buffer->length = nread;
-			soup_message_body_got_chunk (io->read_body, buffer);
-
 			io->read_length -= nread;
 
+			buffer = content_decode (msg, buffer);
+			if (!buffer)
+				continue;
+
+			soup_message_body_got_chunk (io->read_body, buffer);
+
 			if (io->need_content_sniffed) {
 				soup_message_body_append_buffer (io->sniff_data, buffer);
 				soup_buffer_free (buffer);
