@@ -119,6 +119,7 @@ enum {
 	PROP_SERVER_SIDE,
 	PROP_STATUS_CODE,
 	PROP_REASON_PHRASE,
+	PROP_FIRST_PARTY,
 
 	LAST_PROP
 };
@@ -158,6 +159,8 @@ finalize (GObject *object)
 
 	if (priv->uri)
 		soup_uri_free (priv->uri);
+	if (priv->first_party)
+		soup_uri_free (priv->first_party);
 	if (priv->addr)
 		g_object_unref (priv->addr);
 
@@ -576,6 +579,21 @@ soup_message_class_init (SoupMessageClass *message_class)
 				     "The HTTP response reason phrase",
 				     NULL,
 				     G_PARAM_READWRITE));
+
+	/**
+	 * SOUP_MESSAGE_FIRST_PARTY:
+	 *
+	 * Alias for the #SoupMessage:first-party property. (The
+	 * #SoupURI loaded in the application when the message was
+	 * queued.)
+	 **/
+	g_object_class_install_property (
+		object_class, PROP_FIRST_PARTY,
+		g_param_spec_boxed (SOUP_MESSAGE_FIRST_PARTY,
+				    "First party",
+				    "The URI loaded in the application when the message was requested.",
+				    SOUP_TYPE_URI,
+				    G_PARAM_READWRITE));
 }
 
 static void
@@ -612,6 +630,9 @@ set_property (GObject *object, guint prop_id,
 		soup_message_set_status_full (msg, msg->status_code,
 					      g_value_get_string (value));
 		break;
+	case PROP_FIRST_PARTY:
+		soup_message_set_first_party (msg, g_value_get_boxed (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -647,6 +668,9 @@ get_property (GObject *object, guint prop_id,
 	case PROP_REASON_PHRASE:
 		g_value_set_string (value, msg->reason_phrase);
 		break;
+	case PROP_FIRST_PARTY:
+		g_value_set_boxed (value, priv->first_party);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1022,7 +1046,6 @@ soup_message_add_header_handler (SoupMessage *msg,
 				 GCallback    callback,
 				 gpointer     user_data)
 {
-	SoupMessagePrivate *priv;
 	GClosure *closure;
 	char *header_name;
 
@@ -1031,8 +1054,6 @@ soup_message_add_header_handler (SoupMessage *msg,
 	g_return_val_if_fail (header != NULL, 0);
 	g_return_val_if_fail (callback != NULL, 0);
 
-	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
-
 	closure = g_cclosure_new (callback, user_data, NULL);
 
 	header_name = g_strdup (header);
@@ -1237,6 +1258,7 @@ soup_message_cleanup_response (SoupMessage *req)
 		priv->decoders = g_slist_delete_link (priv->decoders, priv->decoders);
 	}
 	priv->msg_flags &= ~SOUP_MESSAGE_CONTENT_DECODED;
+	priv->msg_flags &= ~SOUP_MESSAGE_CERTIFICATE_TRUSTED;
 
 	req->status_code = SOUP_STATUS_NONE;
 	if (req->reason_phrase) {
@@ -1258,6 +1280,13 @@ soup_message_cleanup_response (SoupMessage *req)
  * @SOUP_MESSAGE_OVERWRITE_CHUNKS: Deprecated: equivalent to calling
  * soup_message_body_set_accumulate() on the incoming message body
  * (ie, %response_body for a client-side request), passing %FALSE.
+ * @SOUP_MESSAGE_CONTENT_DECODED: Set by #SoupContentDecoder to
+ * indicate that it has removed the Content-Encoding on a message (and
+ * so headers such as Content-Length may no longer accurately describe
+ * the body).
+ * @SOUP_MESSAGE_CERTIFICATE_TRUSTED: if %TRUE after an https response
+ * has been received, indicates that the server's SSL certificate is
+ * trusted according to the session's CA.
  *
  * Various flags that can be set on a #SoupMessage to alter its
  * behavior.
@@ -1664,3 +1693,55 @@ soup_message_disables_feature (SoupMessage *msg, gpointer feature)
 	}
 	return FALSE;
 }
+
+/**
+ * soup_message_get_first_party:
+ * @msg: a #SoupMessage
+ * 
+ * Returns: the @msg's first party #SoupURI
+ * 
+ * Since: 2.30
+ **/
+SoupURI*
+soup_message_get_first_party (SoupMessage *msg)
+{
+	SoupMessagePrivate *priv;
+
+	g_return_val_if_fail (SOUP_IS_MESSAGE (msg), NULL);
+
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	return priv->first_party;
+}
+
+/**
+ * soup_message_set_first_party:
+ * @msg: a #SoupMessage
+ * @first_party: the #SoupURI for the @msg's first party
+ * 
+ * Sets @first_party as the main document #SoupURI for @msg. For
+ * details of when and how this is used refer to the documentation for
+ * #SoupCookieJarAcceptPolicy.
+ *
+ * Since: 2.30
+ **/
+void
+soup_message_set_first_party (SoupMessage *msg,
+			      SoupURI     *first_party)
+{
+	SoupMessagePrivate *priv;
+
+	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+	g_return_if_fail (first_party != NULL);
+
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+
+	if (priv->first_party) {
+		if (soup_uri_equal (priv->first_party, first_party))
+			return;
+
+		soup_uri_free (priv->first_party);
+	}
+
+	priv->first_party = soup_uri_copy (first_party);
+	g_object_notify (G_OBJECT (msg), SOUP_MESSAGE_FIRST_PARTY);
+}
